#ifndef CACHE_H
#define CACHE_H

#include <QObject>
#include <QDir>
#include <QHash>
#include <vmime/vmime.hpp>

#include "fly-mail-shared_global.h"
#include "letter.h"

#define INCOMING_FOLDER     "Incoming"
#define DRAFTS_FOLDER       "Drafts"
#define SENT_FOLDER         "Sent"
#define ARCHIVE_FOLDER      "Archive"
#define TRASH_FOLDER        "Trash"
#define FULLY_REMOVED_FILE  "fully-removed"

namespace FlyMail {

class FLYMAILSHARED_EXPORT Cache : public QObject
{
    Q_OBJECT
public:
    explicit Cache(QString email, QString server, QObject *parent = nullptr);
    void saveLetterToFolder(Letter *letter, Letter::FolderType folder);

signals:
    void getNewMail(int);
    void seenLetter(Letter * letter);
    void lettersRecieved();
    void deleteMessageFromServer(int);

    void newIncomingLetter(Letter * letter);
    void newDraftLetter(Letter * letter);
    void newSentLetter(Letter * letter);
    void newArchiveLetter(Letter * letter);
    void newTrashLetter(Letter * letter);

    void removeIncomingLetter(int index);
    void removeDraftLetter(int index);
    void removeSentLetter(int index);
    void removeArchiveLetter(int index);
    void removeTrashLetter(int index);

public slots:
    void loadLetter(Letter * letter, int folder);

    void onNewMessageId(int index, QString id);
    void onNewMessage(vmime::shared_ptr <vmime::net::message> msg);

    void archiveMessage(QString id, int index, int folder, Letter *letter);
    void deleteMessage(QString id, int index, int folder, Letter *letter);
    void restoreMessage(QString id, int index, Letter *letter);

private:
    void initialize();
    vmime::shared_ptr <vmime::message> loadLetterFromLocalFolder(QString id, Letter::FolderType folder);
    QString saveMessageToLocalFolder(vmime::shared_ptr <vmime::net::message> msg);
    void saveMessageToFolder(vmime::shared_ptr <vmime::message> msg, Letter::FolderType folder);
    void removeMessageFromFolder(QString id, Letter::FolderType folder, bool fullRemove = false);
    QString getFolderPath(Letter::FolderType folder) const;
    Letter * createLetterFromMsg(vmime::shared_ptr <vmime::net::message> msg);
    Letter * createLetterFromHeader(vmime::shared_ptr <const vmime::header> header);

    void prepareLinks(QString &message);
    void prepareReplies(QString &message);

    QString m_profile;
    QString m_server;

    QDir    m_incomingFolder;
    QDir    m_draftsFolder;
    QDir    m_sentFolder;
    QDir    m_archiveFolder;
    QDir    m_trashFolder;

    QStringList m_allMsgsIdList;
    vmime::shared_ptr <vmime::utility::fileSystemFactory> m_fileSystemFactory;
};

}

#endif // CACHE_H
