#ifndef CONNECTION_H
#define CONNECTION_H

#include <QThread>
#include "server.h"
#include "letter.h"
#include "fly-mail-shared_global.h"

namespace FlyMail {

class FLYMAILSHARED_EXPORT Connection : public QThread
{
    Q_OBJECT
public:
    explicit Connection(QObject *parent = nullptr);

    void setAuthenticator(const bool &in, const QString &login, const QString &password);

signals:
    void error(QString);
    void setUsernameSignal(QString);
    void setPasswordSignal(QString);
    void queryUsername();
    void queryPassword();
    void authFailed(QString);
    void connectionClosed();

    void newMessageId_chain(int, QString);
    void newMessage_chain(vmime::shared_ptr <vmime::net::message>);

public slots:
    void setServer(const bool &in, Server * server);
    void onQueryGetMail(int index = 0);
    void setUsername(QString user);
    void setPassword(QString pass);
    void onSendLetter(Letter * letter);
    void onDeleteMessageFromServer(int index);

private slots:
    void queryUsernameSlot();
    void queryPasswordSlot();

protected:
    void run() override;

private:
    QList <Server *> m_servers;

    Server * m_incomingServer;
    Server * m_outgoingServer;

    vmime::shared_ptr <vmime::net::session> m_session;
    vmime::shared_ptr <Authenticator>       m_incomingAuth;
    vmime::shared_ptr <Authenticator>       m_outgoingAuth;
};

}

#endif // CONNECTION_H
