#ifndef LETTER_H
#define LETTER_H

#include <QObject>
#include <QStringList>
#include <QDateTime>
#include <QVariantList>
#include <vmime/vmime.hpp>
#include "attachment.h"

#define SEPARATOR ":::"

namespace FlyMail {

class FLYMAILSHARED_EXPORT Letter : public QObject
{
    Q_OBJECT
    Q_PROPERTY(bool         parsed      MEMBER parsed)
    Q_PROPERTY(LetterType   letterType  MEMBER letterType   NOTIFY letterTypeChanged)
    Q_PROPERTY(QString      id          MEMBER id           NOTIFY idChanged)
    Q_PROPERTY(QString      from        MEMBER from         NOTIFY fromChanged)
    Q_PROPERTY(QString      replyTo     MEMBER replyTo      NOTIFY replyToChanged)
    Q_PROPERTY(QString      subject     MEMBER subject      NOTIFY subjectChanged)
    Q_PROPERTY(QDateTime    date        MEMBER date         NOTIFY dateChanged)
    Q_PROPERTY(QList<bool>  bodyHtmlList MEMBER bodyHtmlList NOTIFY bodyHtmlListChanged)
    Q_PROPERTY(QStringList  bodyList    MEMBER bodyList     NOTIFY bodyListChanged)
    Q_PROPERTY(QStringList  toList      MEMBER toList       NOTIFY toListChanged)
    Q_PROPERTY(QStringList  copyList    MEMBER copyList     NOTIFY copyListChanged)
    Q_PROPERTY(QVariantList attachList  MEMBER attachList   NOTIFY attachListChanged)

public:
    enum LetterType { LetterUnseen = 0, LetterSeen, LetterSeenWithAttachment };
    enum FolderType { IncomingFolder = 0, DraftsFolder, SentFolder, ArchiveFolder, TrashFolder };
    Q_ENUM(LetterType)
    Q_ENUM(FolderType)

    explicit Letter(QObject *parent = nullptr);
    Letter(const Letter & letter, QObject *parent = nullptr);
    void emitChanged();
    vmime::shared_ptr<vmime::message> toVmimeMessage();

public slots:
    void clear();
    void removeAttachmentAt(int i);
    void setAttachments(QVariantList _attachList);
    void prepareNonQtBody();

public:
    bool        parsed;
    bool        archived; // TODO delete
    bool        removed;  // TODO delete
    LetterType  letterType;
    FolderType  folderType;
    QString     id;
    QString     from;
    QString     replyTo;
    QString     subject;
    QDateTime   date;
    QList<bool> bodyHtmlList;
    QStringList bodyList;
    QStringList toList;
    QStringList copyList;
    QVariantList attachList;
    // TODO embedded images

signals:
    void idChanged();
    void fromChanged();
    void replyToChanged();
    void subjectChanged();
    void dateChanged();
    void bodyHtmlListChanged();
    void bodyListChanged();
    void toListChanged();
    void copyListChanged();
    void letterTypeChanged();
    void attachListChanged();
};

}

#endif // LETTER_H
