#ifndef MAIL_MODEL_H
#define MAIL_MODEL_H

#include <QAbstractListModel>

#include "letter.h"

namespace FlyMail {

class FLYMAILSHARED_EXPORT Model : public QAbstractListModel
{
    Q_OBJECT

public:
    enum MailRoles { LetterRole = Qt::UserRole + 1, FilterRole };
    Q_ENUM(MailRoles)

    explicit Model(QString headersPath, Letter::FolderType folderType, QObject *parent = nullptr);

    QHash<int, QByteArray> roleNames() const override;
    int rowCount(const QModelIndex &parent) const override;
    QVariant data(const QModelIndex &index, int role) const override;
    int unseenLetters();

public slots:
    void onNewLetter(Letter * letter);
    void onSeenLetter(Letter * letter);
    void onRemoveLetter(int index);

signals:
    void unseenLettersChanged(int);
    void checkFolderLimits();

private:
    void initialize();

    int                 m_unseenLetters;
    QList <Letter *>    m_letters; // TODO: limit by 5 last seen msgs
    QString             m_headersPath;
    Letter::FolderType  m_folderType;
};

}

#endif // MAIL_MODEL_H
