#ifndef PROFILEMANAGER_H
#define PROFILEMANAGER_H

#include <QObject>
#include <QList>
#include "profile.h"

namespace FlyMail {

class FLYMAILSHARED_EXPORT ProfileManager : public QObject
{
    Q_OBJECT
    Q_DISABLE_COPY(ProfileManager)
    Q_PROPERTY(int currentProfile       MEMBER m_currentProfile NOTIFY currentProfileChanged)
    Q_PROPERTY(int currentProfileQuery  MEMBER m_currentProfileQuery)
    Q_PROPERTY(int storeMessagesCount   READ getStoreMessagesCount WRITE setStoreMessagesCount NOTIFY storeMessagesCountChanged)
    Q_PROPERTY(int getMailTimerInterval READ getGetMailTimerInterval WRITE setGetMailTimerInterval NOTIFY getMailTimerIntervalChanged)
    Q_PROPERTY(bool deleteFromServer READ getDeleteFromServer WRITE setDeleteFromServer NOTIFY deleteFromServerChanged)
    Q_PROPERTY(QVariantList profiles    READ getProfiles WRITE setProfiles NOTIFY profilesChanged)
public:
    static ProfileManager* instance();
    ~ProfileManager();

    int getStoreMessagesCount() const;
    void setStoreMessagesCount(const int storeMessagesCount);

    int getGetMailTimerInterval() const;
    void setGetMailTimerInterval(int getMailTimerInterval);

    int getDeleteFromServer() const;
    void setDeleteFromServer(int deleteFromServer);

    QVariantList getProfiles() const;
    void setProfiles(const QVariantList &profiles);

signals:
    void currentProfileChanged();
    void profilesChanged();
    void storeMessagesCountChanged();
    void getMailTimerIntervalChanged(int);
    void deleteFromServerChanged();

    void error(QString e);
    void queryUsernameQml();
    void queryPasswordQml();
    void authFailedQml(QString e);
    void trayMessageQml(QString title, QString message);

public slots:
    void addProfile(QString name, QString email, Server * inServer, Server * outServer, QString login = "", QString password = "");
    void changeProfile(int index, QString name, QString email, Server * inServer, Server * outServer);
    void removeProfile(int index);
    void initCurrentProfile(int index);
    void setUsernameQml(QString user, bool save);
    void setPasswordQml(QString pass, bool save);

private slots:
    void queryUsername();
    void queryPassword();
    void authFailed(QString e);
    void onReceiveLetter(Letter *letter);

private:
    void initialize();
    void saveCurrentProfile();
    void setSignals(Profile * profile);
    int getProfileIndex(Profile * profile);

    explicit ProfileManager(QObject *parent = nullptr);
    static ProfileManager* m_pinstance;

    QVariantList m_profiles; // Profile *
    int m_currentProfile;
    int m_currentProfileQuery;

    int m_storeMessagesCount;
    int m_getMailTimerInterval;
    bool m_deleteFromServer;
};

}

#endif // PROFILEMANAGER_H
