#ifndef SERVERMANAGER_H
#define SERVERMANAGER_H

#include <QObject>
#include <QVariant>
#include "fly-mail-shared_global.h"

namespace FlyMail {

class Server;

class FLYMAILSHARED_EXPORT ServerManager : public QObject
{
    Q_OBJECT
    Q_DISABLE_COPY(ServerManager)

    Q_PROPERTY(QVariantList servers     MEMBER m_servers    NOTIFY serversChanged)
    Q_PROPERTY(QStringList typeNames    MEMBER m_typeNames  NOTIFY typeNamesChanged)
    Q_PROPERTY(QStringList protocolNames MEMBER m_protocolNames NOTIFY protocolNamesChanged)
    Q_PROPERTY(QStringList sslNames     MEMBER m_sslNames   NOTIFY sslNamesChanged)
    Q_PROPERTY(QStringList authNames    MEMBER m_authNames  NOTIFY authNamesChanged)

public:
    enum Type { INCOMING_TYPE, OUTGOING_TYPE, UNKNOWN_TYPE };
    enum Protocol { POP3, IMAP, SMTP };
    enum Ssl { AUTO_SSL, NO_SSL, TLS_SSL, STARTTLS_SSL };
    enum Auth { AUTO_AUTH, NO_AUTH, SIMPLE_AUTH, SASL_AUTH };
    Q_ENUM(Type)
    Q_ENUM(Protocol)
    Q_ENUM(Ssl)
    Q_ENUM(Auth)

    static ServerManager* instance();
    ~ServerManager();

    QVariantList getServers() const;

signals:
    void serversChanged();
    void typeNamesChanged();
    void protocolNamesChanged();
    void sslNamesChanged();
    void authNamesChanged();

    void queryCertificateQml(QString str);
    void certificateAnswer(bool);

public slots:
    void addServer(Type type, Protocol protocol, Ssl ssl, Auth auth, QString address/*, QString login*/);
    void changeServer(int index, Type type, Protocol protocol, Ssl ssl, Auth auth, QString address/*, QString login*/);
    void removeServer(int index);
    void restoreServer(const QString &address);

    void certificateAnswerQml(bool answer);
    void onQueryCertificate(QString str);

private:
    void init();
    void saveServerToSettings(Server * server);

    explicit ServerManager(QObject *parent = nullptr);
    static ServerManager* m_pinstance;

    QVariantList m_servers; // Server *

    QStringList m_typeNames;
    QStringList m_protocolNames;
    QStringList m_sslNames;
    QStringList m_authNames;
};

}

#endif // SERVERMANAGER_H
